/**************************************************************************
 *	Copyright (C) 2004  iCapsid.net
 *
 * 	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, 
 *	Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#include "scan_draw.hpp"
#include "fe_node.hpp"

void drawHostSphere();
void drawTinySphere(bool generate = FALSE);
void drawIP(GLuint texIp);

bool dirty = true;
GLuint root_node;

GLfloat vaLabels[] =	{1.0f, 1.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.03f, 0.0f, 0.0f, 
						0.0f, 0.0f, 0.03f, 0.07f, 0.0f, 0.0f, 1.0f, 0.0f, 0.07f, 0.0f };

void ScanDraw::drawScene()
{
	int nodeCount = (int)m_Nodes.size();
	
	if(nodeCount <= 0)
		return;

	float slice =  360.0f/(float)nodeCount;
	float m_Radius = 0;
	float big[2] = { 0, 0};
	float j = 0;

	drawRootNode();

	if(nodeCount > 1)
	{
		float tmp = 0;
		for(vector<FeNode*>::iterator i = m_Nodes.begin(); i != m_Nodes.end(); ++i)
		{
			tmp = (*i)->getRadius();

			if(tmp > big[0])
				big[0] = tmp;
			else if(tmp > big[1])
				big[1] = tmp;
		}
		m_Radius = big[0] + big[1];
	}
	
	glColor3f(0.0f,0.0f,1.0f);

	for(vector<FeNode*>::iterator i = m_Nodes.begin(); i != m_Nodes.end(); ++i)
	{
		glPushMatrix();
		glRotatef(slice*j++,0,1.0f,0);
		glDisable(GL_LIGHTING);
		glBegin(GL_LINES);
			glVertex3f( 0, 0, 0);
			glVertex3f( 0, -0.4f,m_Radius);
		glEnd();
		glEnable(GL_LIGHTING);

		glTranslatef(0,-0.4f,m_Radius);
		(*i)->draw();
		glPopMatrix();
	}
}

void ScanDraw::draw(GLenum mode)
{
	glTranslatef(0.0f,y_Trans,z_Trans);
	glRotatef(rtri,0.0f,-0.5f,0.0f);
	glRotatef(x_Rot, 1, 0, 0);
	glRotatef(y_Rot, 0, 1, 0);

	drawScene();

	if(rotateOn)
		rtri+=rotSpeed;
}



void ScanDraw::generate()
{
}

void ScanDraw::drawRootNode()
{
	// Implement correct earth tilt. : ) 
	glPushMatrix();
		glRotatef(m_Config->m_RootTilt, 1.0f, 0.0f, 0.0f);
		glEnable(GL_LIGHTING);
		glEnable(GL_TEXTURE_2D);
			drawHostSphere();
		glDisable(GL_TEXTURE_2D);
		glDisable(GL_LIGHTING);
		glColor3f(0.0f, 0.0f, 0.5f);
		glBegin(GL_LINES);
			glVertex3f( 0, 0.2f, 0);
			glVertex3f( 0, -0.2f,0.0f);
		glEnd();
	glPopMatrix();
}

void ScanDraw::addNode(FeNode* node)
{
	if(node != NULL)
		m_Nodes.push_back(node);
}

ScanDraw::ScanDraw()
{
	m_dList		= glGenLists(1);
	rotateOn	= FALSE;
	fpsOn		= FALSE;
	rtri		= 0;
	x_Rot		= 0;
	y_Rot		= 0;
	rotSpeed	= 0.5f;
	m_Dirty		= TRUE;
	y_Trans		= 0.6f;
	z_Trans		= -2.0f;
}

ScanDraw::~ScanDraw()
{
	glDeleteLists(m_dList, 1);

	while(!m_Nodes.empty())
	{
		delete *(m_Nodes.begin());
		m_Nodes.erase(m_Nodes.begin());
	}
}

PortDraw::PortDraw(int port, STATE state, PROTOCOL prot)
{
	m_Port = port;
	m_State = state;
	m_Protocol = prot;
	m_Dirty = TRUE;
	m_Service = NULL;
	portTexture = 0;
}

PortDraw::~PortDraw()
{
	glDeleteTextures(1, &portTexture);
	
	free(m_Service);
}

void PortDraw::setService(char* service)
{
	if(service == NULL)
		return;

	m_Service = (char*) malloc(strlen(service) + 1);
	strcpy(m_Service, service);
	
	serviceTexture = m_TexStore->getServiceTexture(service);
}
void PortDraw::draw(GLenum mode)
{
	if(this->portTexture == 0)
		generate();
	
	glPushMatrix();
		glRotatef(90, 0, 0, 1);
		glTranslatef(0.03f, -0.035f, 0.075f);

//We can move this routine to HostDraw, change the textures to one big texture, 
// resign tex coordinates so that only sub-textures are taken
// draw the whole tree using one big ass vertex array with GL_QUADS 

		glEnable(GL_TEXTURE_2D);
		glEnable(GL_LIGHTING);
		glBindTexture(GL_TEXTURE_2D, portTexture);
		vaLabels[13] = vaLabels[18] = 0.07f;
		glInterleavedArrays(GL_T2F_V3F, 0, vaLabels);
		glDrawArrays(GL_QUADS, 0, 4);

		if(m_Service)
		{
			int servCount = (int)strlen(m_Service);
			vaLabels[13] = vaLabels[18] = (((0.07f)/5)*servCount);
			glBindTexture(GL_TEXTURE_2D, serviceTexture);
			glTranslatef(0.0f, -(0.01f+((((0.07f)/5)*servCount))), 0.0f);
			glInterleavedArrays(GL_T2F_V3F, 0, vaLabels);
			glDrawArrays(GL_QUADS, 0, 4);
		}

	glPopMatrix();
	glDisable(GL_TEXTURE_2D);

	drawTinySphere(TRUE);
}




void PortDraw::generate()
{
	SDL_Surface* tmpSurf = NULL;

	tmpSurf = m_Config->m_glUtil->createTextSurfaceGL(m_Port);

	if(tmpSurf == NULL)
		return;

	// Create a texture for Port
	glGenTextures(1, &portTexture);
    glBindTexture(GL_TEXTURE_2D, portTexture);   // 2d texture (x and y size)
//    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR); // scale linearly when image bigger than texture
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR); // scale linearly when image smalled than texture
    glTexImage2D(GL_TEXTURE_2D, 0, 3, tmpSurf->w, tmpSurf->h, 0, GL_RGBA, GL_UNSIGNED_BYTE, tmpSurf->pixels);

	SDL_FreeSurface(tmpSurf);
}

void FeDraw::drawSelected()
{
	glPushMatrix();
		glDisable(GL_TEXTURE_2D);
		glDisable(GL_LIGHTING);
		glTranslatef(0.0f, 0.08f, 0.0f);
		glColor3f(1.0f, 0.0f, 0.0f);
		drawSphere(0.02f);
		glEnable(GL_TEXTURE_2D);
		glEnable(GL_LIGHTING);
	glPopMatrix();
}
