/**************************************************************************
 *	Copyright (C) 2004  iCapsid.net
 *
 * 	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, 
 *	Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#include "tex_cache.hpp"
#include <iostream>

using namespace std;

TextureStore::TextureStore()
{
	
}

TextureStore::~TextureStore()
{
//	map<OS_FAMILY::OS,GLuint>::const_iterator iter;
	
   	while(!m_texOSFam.empty())
	{
		glDeleteTextures(1, &(m_texOSFam.begin()->second));
		m_texOSFam.erase(m_texOSFam.begin());
    }

//	map<string,GLuint>::const_iterator iter2;

   	while(!m_texPorts.empty())
	{
		glDeleteTextures(1, &(m_texPorts.begin()->second));
		m_texPorts.erase(m_texPorts.begin());
    }
}

void TextureStore::addOSTexture(OS_FAMILY::OS osid, GLuint texId)
{
	m_texOSFam[osid] = texId;
}

GLuint TextureStore::getOSTexture(OS_FAMILY::OS osid)
{
	if(m_texOSFam.size() > 0)
		return m_texOSFam[osid];
	else
		return 0;
}

void TextureStore::loadOSTexture(OS_FAMILY::OS osid, const char* name)
{
	GLuint tmpTex = 0;
	SDL_Surface* tmpSurf = 	m_glUtil->LoadBMP(name, FALSE, FALSE);
	glGenTextures(1, &tmpTex);

	glBindTexture(GL_TEXTURE_2D, tmpTex);   // 2d texture (x and y size)
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,GL_LINEAR); 
    glTexImage2D(GL_TEXTURE_2D, 0, 3, tmpSurf->w, tmpSurf->h, 0, GL_RGB, GL_UNSIGNED_BYTE, tmpSurf->pixels);

	this->addOSTexture(osid, tmpTex);
	SDL_FreeSurface(tmpSurf);
}


void TextureStore::addServiceTexture(const char* ps)
{
	GLuint texId;
	// Create a texture for Port

	glGenTextures(1, &texId);


	unsigned int sz = (unsigned int)(64*(strlen(ps)))/3;

	if(sz <= 32)
		sz = 32;
	else if(sz <= 64)
		sz = 64;
	else if(sz <= 128)
		sz = 128;	
	else if(sz <= 256)
		sz = 256;
	else
		sz = 512;


	SDL_Surface* tmpSurf = m_glUtil->createTextSurfaceGL(ps, sz, 64, 40, TRUE);

	if(!tmpSurf)
	{
		cout<<"TC:addServiceTex -) Failed tgen "<<ps <<endl;
		return;	
	}

    	glBindTexture(GL_TEXTURE_2D, texId);
//    	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR); // scale linearly when image bigger than texture
    	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR); // scale linearly when image smaller than texture
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);

    	glTexImage2D(GL_TEXTURE_2D, 0, 3, tmpSurf->w, tmpSurf->h, 0, GL_RGBA, GL_UNSIGNED_BYTE, tmpSurf->pixels);

	SDL_FreeSurface(tmpSurf);

	m_texPorts[ps] = texId;
}


GLuint TextureStore::getServiceTexture(const char* ps)
{
	GLuint tmp = m_texPorts[ps];

	if(tmp == 0)
	{
		addServiceTexture(ps);
		tmp = m_texPorts[ps];
	}

	return tmp;
}
